# -*- encoding: utf-8 -*-

require 'sn_item'
require 'mysql2'
require 'pg'
require 'sqlite3'
require 'active_record'
require 'mechanize'
require 'date'
require 'curb'
require 'zlib'
require 'digest/sha2'
require 'xml'
require 'nokogiri'
require 'readability'

## 
#
# A crawler class
#
# @example
#   require 'sn_crawler'
#   
#   url = "http://vnexpress.net/rss"
#   c = SNCrawler.new(url,"VNE","/channel/item:[title,description,pubDate,link]",nil)
#   c.get_links(true)
#   # => ["vnexpress.net/rss/tin-moi-nhat.rss",...]
#
#   c.get_news(true)
#   # => City in Jordan welcomes ISIS
#   #    Image: []
#   #    Now inserting City in Jordan welcomes ISIS
#
class SNCrawler
  ## 
  # 
  # Initialize parameters
  # 
  # source => The source URL
  #
  # name => Some lovely names
  #
  # structure =>
  #
  # structure format: /path/to/channel/item_name:[item_attributes_name]
  #
  # item_attributes_name[0] => title of the page
  #
  # item_attributes_name[1] => description of the page
  #
  # item_attributes_name[2] => publicity time of the page
  #
  # item_attributes_name[3] => link to the page
  #
  # For example: /channel/item:[title,description,pubDate,url]
  #
  # db_conf => activerecord settings
  #
  # limit the number of news that you want
  #
  def initialize(source = "", name = "", structure = "", db_conf = {}, limit = 100)
    ## The rss source's url
    @source = source
    @source_name = name
    @structure = structure
    ## Crawled urls will be stored here
    @url = []
    @agent = Mechanize.new
    ## Establish a connection to DB server
    if !db_conf.nil? then
      ActiveRecord::Base.establish_connection(db_conf)
      @use_db = true
    else
      @use_db = false
    end
    @limit = limit
  end

  ##
  #
  # Create table for our gem
  #
  # @example
  #   c.create_table("engine=MROONGA",true)
  #
  def create_table(options = "", verbose = true)
    begin
      ActiveRecord::Migration.class_eval do
        create_table :sn_news, :options => options do |t|
          t.string :lang
          t.string  :title
          t.text :description
          t.text :content
          t.string :link
          t.string :images
          t.datetime :pubtime
          
          t.timestamps
        end
        
        add_index :sn_news, [:title], :unique => true, :name => "unique_title_on_news"
      end
    rescue => e
      if verbose then
        puts "Error(s): #{e.to_s}"
      end
      return false
    end
    return true
  end

  ## 
  #
  # Get urls from a source url
  #
  def get_links(verbose = false)
    page = @agent.get(@source)

    page.links_with(:href => /\.(rss|xml)/).each do |link|
      src = ""
      if !link.href.include? "http"
        src = URI.parse(@source).host + link.href 
      else 
        src = link.href
      end
      existed = 0
      @url.each do |u|
        if u == src
          existed = 1
        end
      end
      if existed == 0 && src.length <= 50
        @url << src
      end
    end

    if verbose then
      puts @url.to_s
    end
  end

  ## 
  #
  # Set Urls
  #
  def set_url(url = [])
    @url = url
  end

  ## 
  #
  # Clear urls
  #
  def clear_url
    @url = []
  end

  ## 
  #
  # Get news from urls
  #
  # Note that you have to run this method ONLY after running get_links and create_table(in case of using DB)
  #
  def get_news(verbose = false)
    count = 0
    channel_path = "."
    structure_path = @structure.split(/\//)
    length = structure_path.length
    for i in 0..(length - 2) do
      channel_path += "/" + structure_path[i]
    end
    item_structure = structure_path[length - 1]
    item_sts = item_structure.split(/:/)
    ## Tag names
    item_tag = item_sts[0]
    item_sts2 = item_sts[1].gsub(/(\[|\])/,'').split(/,/)
    title_tag = item_sts2[0]
    des_tag = item_sts2[1]
    pubdate_tag = item_sts2[2]
    link_tag = item_sts2[3]
    @url.each do |u|
      request = Curl.get(u.to_s)
      begin
        source = XML::Parser.string(request.body_str)
        root_content = source.parse
        ## Find all channels
        channels = root_content.root.find(channel_path)
        ## For each channel processing the data
        channels.each do |c|
          lang_field = c.find_first('language')
          if lang_field.nil? then
            lang = "en_US"
          else 
            lang = lang_field.content.to_s
          end
          items = c.find(item_tag)
          puts item_tag
          items.each do |i|
            title = i.find_first(title_tag).content
            title = title.gsub("'","")
            puts title
            description = i.find_first(des_tag).content
            description = description.gsub("'","")
            doc = Nokogiri::HTML(description)
            img_url = []
            doc.search('img').each do |img_tag|
              img_url << img_tag.attributes['src'].value
            end
            puts "Image: #{img_url.to_s}"
            link = i.find_first(link_tag).content.gsub(" ","")
            pub_date = Time.strptime(i.find_first(pubdate_tag).content,"%A, %d %B %Y %H:%M:%S %Z")
            i_source = Curl.get(link).body_str
            content = Readability::Document.new(i_source).content
            content = content.gsub("'","").force_encoding("UTF-8")
            puts "Now inserting #{title}"
            time_now = Time.now.strftime("%Y-%m-%d %H:%M:%S")
            begin
              if @use_db then
                SNItem.create(
                :lang => lang,
                :title => title,
                :description => description,
                :link => link,
                :pubtime => pub_date.strftime("%Y-%m-%d %H:%M:%S"),
                :content => content,
                :images => img_url.to_s,
                :created_at => time_now,
                :updated_at => time_now
                )
              else
                if verbose then
                  puts "You do not use DB"
                end
              end
            rescue => e
              if verbose then
                puts "Error(s): #{e.to_s}"
              end
            else
            end
            count = count + 1
            if verbose then
              puts "Got #{count} news"
            end
          end
        end
      rescue => e
        if verbose then
          puts "Error: #{e}"
        end
      end
      if count >= @limit then
        break
      end
    end

    if verbose then
      puts "We got #{count} news today."
    end
  end

  def finalize
    ActiveRecord::Base.connection.close
  end
end